@@ -16,13 +16,14 @@
 import re
 import os
 import sys
-import tyrs
 import time
+import tyrs
 import signal                   # resize event
 import curses
+from user import User
 from timeline import Timeline
 from message import FlashMessage
-from utils import html_unescape, encode
+from utils import html_unescape, encode, get_source, get_urls
 
 class Interface(object):
     ''' All dispositions in the screen
@@ -45,12 +46,13 @@ class Interface(object):
         self.conf       = tyrs.container['conf']
         self.timelines  = tyrs.container['timelines']
         self.buffers    = tyrs.container['buffers']
+        tyrs.container.add('interface', self)
+        self.api.set_interface()
         self.resize_event     = False
         self.regex_retweet     = re.compile('^RT @\w+:')
         self.refresh_token    = False
         self.buffer           = 'home'
         self.charset = sys.stdout.encoding
-        self.api.set_interface(self)
         # resize event
         signal.signal(signal.SIGWINCH, self.sigwinch_handler)
         self.init_screen()
@@ -165,6 +167,7 @@ class Interface(object):
            make sure to set self.buffer before
         '''
         self.set_max_window_size()
+        self.set_date()
         try:
             if not self.refresh_token:
                 timeline = self.select_current_timeline()
@@ -191,6 +194,9 @@ class Interface(object):
         except curses.error:
             pass
 
+    def set_date(self):
+        self.date = time.strftime("%d %b", time.gmtime())
+
     def select_current_timeline(self):
         return self.timelines[self.buffer]
 
@@ -266,7 +272,10 @@ class Interface(object):
 
         # The content of the tweets is handle
         # text is needed for the height of a panel
-        header  = self.get_header(status)
+        try:
+            header  = self.get_header(status)
+        except UnicodeDecodeError:
+            header = 'encode error'
 
         # We get size and where to display the tweet
         size = self.get_size_status(status)
@@ -281,7 +290,10 @@ class Interface(object):
         panel = curses.newpad(height, length)
 
         if self.conf.params['tweet_border'] == 1:
-            panel.border(0)
+            if self.conf.params['old_skool_border']:
+                panel.border('|','|','-','-','+','+','+','+')
+            else:
+                panel.border(0)
 
         # Highlight the current status
         if timeline.current == i:
@@ -326,23 +338,28 @@ class Interface(object):
         words = text.split(' ')
         margin = self.conf.params['margin']
         padding = self.conf.params['padding']
+        myself = self.api.myself.screen_name
         curent_x = padding
         line = 1
+
+        hashtag = encode('#')
+        attag = encode('@')
+
+
         for word in words:
-            word = word.encode(self.charset)
+            word = encode(word)
             if curent_x + len(word) > self.maxyx[1] - (margin + padding)*2:
                 line += 1
                 curent_x = padding
 
             if word != '':
                 # The word is an HASHTAG ? '#'
-                if word[0] == '#':
+                if word[0] == hashtag:
                     panel.addstr(line, curent_x, word, self.get_color('hashtag'))
                 # Or is it an 'AT TAG' ? '@'
-                elif word[0] == '@':
-                    name = self.api.myself.screen_name
+                elif word[0] == attag:
                     # The AT TAG is,  @myself
-                    if word == '@'+name or word == '@'+name+':':
+                    if word == attag + myself or word == attag + myself+ encode(':'):
                         panel.addstr(line, curent_x, word, self.get_color('highlight'))
                     # @anyone
                     else:
@@ -391,12 +408,14 @@ class Interface(object):
         @return string: readeable time
         '''
         if self.conf.params['relative_time'] == 1 and self.buffer != 'direct':
-            hour =  status.GetRelativeCreatedAt()
+            result =  status.GetRelativeCreatedAt()
         else:
             hour = time.gmtime(status.GetCreatedAtInSeconds() - time.altzone)
-            hour = time.strftime('%H:%M', hour)
+            result = time.strftime('%H:%M', hour)
+            if time.strftime('%d %b', hour) != self.date:
+                result += time.strftime(' - %d %b', hour)
 
-        return hour
+        return result
 
     def get_header(self, status):
         nick = self.get_nick(status)
@@ -411,8 +430,19 @@ class Interface(object):
         if self.is_reply(status):
             header += u'\u2709 '
 
+        source = self.get_source(status)
+        if self.conf.params['source'] and source:
+            header += ' (%s)' % source
+
         return encode(header)
 
+    def get_source(self, status):
+        if hasattr(status, 'source'):
+            source = get_source(status.source)
+        else:
+            source = ''
+        return source
+
     def get_nick(self, status):
         if hasattr(status, 'user'):
             nick = status.user.screen_name
@@ -465,12 +495,6 @@ class Interface(object):
         timeline = self.select_current_timeline()
         return timeline.statuses[timeline.current]
 
-    def get_urls(self):
-        '''
-        @return array of urls find in the text
-        '''
-        return re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', self.current_status().text)
-
     def get_color(self, color):
         '''Return the curses code, with bold if enable of the color
            given in argument of the function
@@ -488,6 +512,16 @@ class Interface(object):
             if timeline.current >= timeline.last:
                 timeline.first += 1
             timeline.current += 1
+        else:
+            self.lazzy_load()
+
+    def lazzy_load(self):
+        timeline = self.select_current_timeline()
+        timeline.page += 1
+        statuses = self.api.retreive_statuses(self.buffer, timeline.page)
+        timeline.append_old_statuses(statuses)
+        timeline.first += 1
+        timeline.current += 1
 
     def move_up(self):
         timeline = self.select_current_timeline()
@@ -501,10 +535,18 @@ class Interface(object):
         timeline = self.select_current_timeline()
         timeline.current = timeline.last
 
+    def back_on_top(self):
+        timeline = self.select_current_timeline()
+        timeline.current = 0
+        timeline.reset()
+
     def openurl(self):
-        urls = self.get_urls()
+        urls = get_urls(self.current_status().text)
         for url in urls:
             try:
                 os.system(self.conf.params['openurl_command'] % url)
             except:
                 pass 
+
+    def current_user_info(self):
+        User(self.current_status().user)
